#include "sut.h"

/****************************************************************************
  Perform handshaking
 ****************************************************************************/

/* 
   Perform the handshake. 
   Not only does this make sure that the process is alive, but it propagates
   the values of SUT_Debug and SUT_Options.  Currently requires
   a HOMOGENEOUS system 

   Eventually needs to either send or get the node list array.

   Need to split; master SENDS args, slave RECVS args
 */

int SUT_Handshake_master( rdfd, wrtfd )
int rdfd, wrtfd;
{
struct timeval timer;
fd_set rset;
char   buf[10];
int    parms[2];
SUT_Header msg;

if (SUT_Debug & SUT_HANDSHAKE) {
    fprintf( stderr, "[%d] Waiting for alive\n", SUT_Mynode );
    }
/* Read the ready message from the created child */
while (1) {
    FD_ZERO(&rset);
    FD_SET(rdfd, &rset);
    timer.tv_sec = SUT_TIMEOUT_VAL;
    timer.tv_usec = 0;
    switch(select( rdfd+1, &rset, (fd_set*)NULL, (fd_set*)NULL, &timer )) {
	case -1:
	if (errno == EINTR) continue;
	perror("select");
	return SUT_SELECT_FAILED;
	
	case 0:
	if (SUT_Debug & SUT_HANDSHAKE)
	    fprintf( stderr, "[%d] Select timed out waiting for alive\n",
		    SUT_Mynode );
	return SUT_CHILD_UNRESPONSIVE;
	    
	default:
	if (SUT_Debug & SUT_HANDSHAKE)
	    fprintf( stderr, 
		    "[%d] Handshake received; checking for correctness\n", 
		    SUT_Mynode );
	/* All is well.  Send the proceed and continue */
	READ( rdfd, &msg, sizeof(SUT_Header) );  
	if (!msg.op == SUT_ALIVE)
	    /* Got the wrong response.  */
	    return SUT_CHILD_CONFUSED;
	else {
	    if (SUT_Debug & SUT_HANDSHAKE)
		fprintf( stderr, "[%d] Returning proceed\n", SUT_Mynode );
	    msg.op = SUT_PROCEED;
	    WRITE( wrtfd, &msg, sizeof(SUT_Header) );
	    parms[0] = SUT_Debug;
	    parms[1] = SUT_Options;
	    WRITE( wrtfd, parms, 2 * sizeof(int) );
	    }
	return SUT_SUCCESS;
	}  /* End of switch on select for alive message */
    }
return SUT_SUCCESS;
}

int SUT_Handshake_slave( rdfd, wrtfd )
int rdfd, wrtfd;
{
struct timeval timer;
fd_set rset;
char   buf[10];
int    parms[2];
SUT_Header msg;

if (SUT_Debug & SUT_HANDSHAKE)
    fprintf( stderr, "[%d] Sending alive to parent\n", SUT_Mynode );
msg.op = SUT_ALIVE;
WRITE( wrtfd, &msg, sizeof(SUT_Header) );
READ( rdfd, &msg, sizeof(SUT_Header) );
if (msg.op != SUT_PROCEED) 
    return SUT_PARENT_CONFUSED;
READ( rdfd, parms, 2 * sizeof(int) );
SUT_Debug	= parms[0];
SUT_Options	= parms[1];
SUT_Mynode  = SUT_Mynodenum();

return SUT_SUCCESS;
}

int SUT_Handshake( rdfd, wrtfd, is_master )
int rdfd, wrtfd, is_master;
{
struct timeval timer;
fd_set rset;
char   buf[10];
int    parms[2];
SUT_Header msg;

if (is_master) 
    return SUT_Handshake_master( rdfd, wrtfd );
else 
    return SUT_Handshake_slave( rdfd, wrtfd );
}

/*
   We also need a non-blocking test and handshake.  Somethink like

   fd_set fd_waiting;
   
   n_waiting = select( , fd_waiting, ..., timeout = 0 or SUT_TIMEOUT );

   switch (n_waiting) {
   case -1: check for EINTR
   case 0:  timed out (if not timeout = 0, all dead, call code to
            try different processes
   default:
   for (i=0; i<nalive; i++) 
       if (FD_ISSED(i,fd_waiting)) {
          Handshake with i
	  conn->state = SUT_ALIVE;
	  FD_CLR(i,fd_waiting);
	  }

 */

